﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;

using Model;
using View;
using System.Threading;
using System.Diagnostics;

namespace Controller
{
    /// <summary>
    /// Handles all user mouse input.
    /// </summary>

    /**
    * This class performs appropriate actions in response to user mouse input.
    */
    static class MouseHandler
    {
        private static MouseState lastMouseState;          // this state allows for detection of a single click (not click and hold)
        private static MouseState currentMouseState;
        private static TurnState turnState = TurnState.Wait;
        

        /**
        Performs appropriate actions in response to mouse input.
            \param graph Graph representing the current game map.
        */
        public static void updateMouse(Graph graph)
        {
            lastMouseState = currentMouseState;
            currentMouseState = Mouse.GetState();

            // valid coordinates for the mouse (ie. must be inside the game window)
            bool validX = currentMouseState.X < GameState.SCREEN_WIDTH && currentMouseState.X > 0;
            bool validY = currentMouseState.Y < GameState.SCREEN_HEIGHT && currentMouseState.Y > 0;

            // if unit is selected when right-clicked, deselect it
            if (lastMouseState.RightButton == ButtonState.Released
                && currentMouseState.RightButton == ButtonState.Pressed
                && validX && validY && GameState.playableUnitSelected)
            {
                setSelectedUnit(null, false);
                turnState = TurnState.Wait;
                GameState.unitToAttack = null;
                GameState.attackConfirmOpen = false;
                GameState.dropDownMenuOpen = false;
            }

            // checks for single mouse left click inside the game window
            if (lastMouseState.LeftButton == ButtonState.Released
                && currentMouseState.LeftButton == ButtonState.Pressed
                && validX && validY)
            {
                // do not react to mouse input if animation is on screen
                if (GameState.isAnimating)
                {
                    return;
                }

                Vector2 mouseClickCoordinates = new Vector2(currentMouseState.X, currentMouseState.Y);   // mouse click coordinates
                Node nodeClickedOn = graph.getNode(mouseClickCoordinates);

                if (GameState.gameOver)
                {
                    if ((mouseClickCoordinates.X > 370 & mouseClickCoordinates.X < 556) & (mouseClickCoordinates.Y>300&mouseClickCoordinates.Y<396)){
                        GameState.exitGameClicked = true;
                    }
                }

                #region Check if a unit is clicked

                // if player clicks after unit is already selected ...
                if (GameState.playableUnitSelected)
                {
                    // determine if player has clicked on a menu button; if so, change turnState
                    if (getMenuButtonClicked(mouseClickCoordinates) != null)
                    {
                        Button menuButton = getMenuButtonClicked(mouseClickCoordinates);
                        Debug.WriteLine(getMenuButtonClicked(mouseClickCoordinates).getButtonType());
                        buttonAction(menuButton, graph);
                        return;
                    }

                    // if user clicks on a valid end node, move to it
                    if (turnState == TurnState.Move)
                    {
                        Game.Instance.Tick();

                        // set variables for path finding
                        Node startNode = graph.getNode(GameState.selectedUnit.Position);
                        Node endNode = graph.getNode(mouseClickCoordinates);
                        LinkedList<Node> path = GameFunction.pathFinder(graph, GameState.selectedUnit, startNode, endNode);

                        // if path finder returns a non-null path, then end node is valid
                        if (path != null)
                        {
                            GameState.selectedUnit.getButtonOfType(ButtonType.Move).Active = false;   // move button no longer active
                            updateUnitPosition(graph, mouseClickCoordinates, path);
                            GameState.beforeMove = false;
                            turnState = TurnState.Wait;
                        }
                        else
                        {
                            setSelectedUnit(null, false);       // if user clicks on invalid end node, deselect it
                            turnState = TurnState.Wait;
                        }
                    }

                    if (turnState == TurnState.Attack) // if a unit is clicked after attack is clicked
                    {
                        // gets enemy unit where user clicked
                        Unit unit = getUnitOnNodeClicked(graph.getNode(mouseClickCoordinates), mouseClickCoordinates, GameState.enemyPlayer);
                        if (unit != null && GameFunction.isEnemyUnitInRange(graph, GameState.selectedUnit, unit))
                        {
                            GameState.unitToAttack = unit;      // set state of attacked unit
                            GameState.attackConfirmOpen = true; // opens attack confirmation
                        }
                    }
                }

                // if player clicks when a unit is not selected ...
                else
                {
                    // if there is a playable, player-owned unit on position clicked, set selected unit status
                    Unit unit = getUnitOnNodeClicked(graph.getNode(mouseClickCoordinates), mouseClickCoordinates, GameState.currentPlayer);
                    if (unit != null && !GameFunction.hasUnitFinishedActions(unit))
                    {
                        setSelectedUnit(unit, true);
                        GameFunction.setMovableNodes(graph, unit);
                        GameState.dropDownMenuOpen = true;
                        if (unit.getButtonOfType(ButtonType.Move).Active)
                        {
                            GameState.beforeMove = true;
                        }
                    }
                }

                #endregion
            }
        }

        /**
        Sets selection of unit state inside GameState.
            \param unit Unit to set selection of.
            \param selected Indicates whether or not a selection is occuring.
        */
        private static void setSelectedUnit(Unit unit, bool selected)
        {
            GameState.playableUnitSelected = selected;
            GameState.selectedUnit = unit;
        }

        /**
        If unit exists where user clicked (that belongs to player), return it; else, return null.
            \param clickedNode Node where user has clicked.
            \param positionClicked position (by node) of where the user has clicked.
            \param player Player that is currently moving.
        */
        private static Unit getUnitOnNodeClicked(Node clickedNode, Vector2 positionClicked, Player player)
        {
            for (int i = 0; i < player.getNumOfUnits(); i++)
            {
                Unit unit = player.getUnits().ElementAt(i);
                int unitX = unit.Position.Item1;
                int unitY = unit.Position.Item2;
                int clickedX = (int)Math.Floor(positionClicked.X / 32);
                int clickedY = (int)Math.Floor(positionClicked.Y / 32);
                if (unitX == clickedX && unitY == clickedY)
                {
                    return unit;
                }
            }
            return null;
        }

        /**
        Updates the unit's position to the clicked position.
            \param graph Graph representing the current game map.
            \param position Position (by node) to move the unit to.
            \param path Path to move the unit along.
        */
        private static void updateUnitPosition(Graph graph, Vector2 position, LinkedList<Node> path)
        {
            Unit unit = GameState.selectedUnit;
            GameState.isAnimating = true;
            // updates the unit's position to each node in the path (node by node)
            foreach (Node node in path)
            {
                animateUnitPosition(graph, unit, node);
                unit.Position = (new Tuple<int, int>(node.getPositionX(), node.getPositionY()));
                node.unitOnNode = (unit);
                GameState.dropDownMenuOpen = true;

            }
            GameState.isAnimating = false;
        }

        private static void attackAnimation(Direction direction, Unit unit)
        {
            GameState.isAnimating = true;
            float originalLocationX = unit.PixelCoordinates.X;
            float originalLocationY = unit.PixelCoordinates.Y;
            #region Attack Right
            if (direction == Direction.Right) //attack right
            {
                for (float i = originalLocationX; i <= originalLocationX + 8; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i > originalLocationX; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Left
            else if (direction == Direction.Left) //attack left
            {
                for (float i = originalLocationX; i >= originalLocationX - 8; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i < originalLocationX; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Down
            else if (direction == Direction.Down) //attack down
            {
                for (float i = originalLocationY; i <= originalLocationY + 8; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y + 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.Y; i > originalLocationY; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y - 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Up
            else if (direction == Direction.Up) //attack up
            {
                for (float i = originalLocationY; i >= originalLocationY - 8; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y - 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.Y; i < originalLocationY; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y + 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            GameState.isAnimating = false;
        }

        // Moves unit to move from to different nodes
        private static void animateUnitPosition(Graph graph, Unit unit, Node node)
        {
            int nodePixelX = node.getPositionX() * 32;
            int nodePixelY = node.getPositionY() * 32;
            int pixelDifference = 4;
            graph.getNode(unit.Position).unitOnNode = (null);

            while (unit.PixelCoordinates.X != nodePixelX)
            {
                if (unit.PixelCoordinates.X < nodePixelX)
                {
                    unit.animate(Direction.Right);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + pixelDifference, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
                else
                {
                    unit.animate(Direction.Left);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - pixelDifference, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
            }

            while (unit.PixelCoordinates.Y != nodePixelY)
            {
                if (unit.PixelCoordinates.Y < nodePixelY)
                {
                    unit.animate(Direction.Down);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y + pixelDifference);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
                else
                {
                    unit.animate(Direction.Up);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y - pixelDifference);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
            }
        }

        // returns a menu button that was clicked; if no menu button was clicked, returns null
        private static Button getMenuButtonClicked(Vector2 mouseCoordinates)
        {
            Unit unit = GameState.selectedUnit;
            Button[] menuButtons = unit.getButtons();
            int clickX = (int)mouseCoordinates.X;
            int clickY = (int)mouseCoordinates.Y;

            for (int i = 0; i < menuButtons.Length; i++)
            {
                int buttonX = (int)menuButtons[i].getPixelCoordinates().X;
                int buttonY = (int)menuButtons[i].getPixelCoordinates().Y;
                if (buttonX <= clickX && clickX < buttonX + 128 && buttonY <= clickY && clickY < buttonY + 32)
                {
                    if (GameState.dropDownMenuOpen)
                    {
                        return menuButtons[i];
                    }
                }
            }

            int ButtonX = (int)unit.getButtonOfType(ButtonType.AttackConfirm).getPixelCoordinates().X;
            int ButtonY = (int)unit.getButtonOfType(ButtonType.AttackConfirm).getPixelCoordinates().Y;
            if (ButtonX+90 <= clickX && clickX < ButtonX + 214 && ButtonY+127 <= clickY && clickY < ButtonY + 172)
            {
                if (GameState.attackConfirmOpen)
                {
                    return unit.getButtonOfType(ButtonType.AttackConfirm);
                }
            }
            return null;
        }

        private static bool findAttackType(Unit unit1)
        {
            if ((unit1.getClass() == UnitType.Warrior) || (unit1.getClass() == UnitType.Archer))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        // sets which menu option is selected
        private static void buttonAction(Button button, Graph graph)
        {
            // take action corresponding to which button was clicked
            Unit unit = GameState.selectedUnit;
            Unit unit2 = GameState.unitToAttack;
            switch (button.getButtonType())
            {
                case ButtonType.Attack:         // if attack clicked
                    turnState = TurnState.Attack;
                    GameState.dropDownMenuOpen = false;    // close the dropdownmenu when selecting who to attack
                    break;
                case ButtonType.AttackConfirm:         // if confirm attack clicked
                    button.Active = false;
                    unit.getButtonOfType(ButtonType.Move).Active = false;     // move button is no longer active
                    unit.getButtonOfType(ButtonType.Attack).Active = false;   // attack button is no longer active
                    int attackDirection = 0;
                    int counterAttackDirection = 0;
                    //Gets attack anaimation direction
                    #region Attack animation Direction
                    Direction attackDir = Direction.Up;
                    Direction counterAttackDir = Direction.Up;
                    if (unit.Position.Item1 < unit2.Position.Item1) //attack right
                    {
                        attackDirection = 0;
                        counterAttackDirection = 1;
                        attackDir = Direction.Right;
                        counterAttackDir = Direction.Left;
                        unit.setCurrentFrame(7);
                        unit2.setCurrentFrame(4);
                    }
                    else if (unit.Position.Item1 > unit2.Position.Item1) //attack left
                    {
                        attackDirection = 1;
                        counterAttackDirection = 0;
                        attackDir = Direction.Left;
                        counterAttackDir = Direction.Right;
                        unit.setCurrentFrame(4);
                        unit2.setCurrentFrame(7);
                    }
                    else if (unit.Position.Item2 < unit2.Position.Item2) //attack down
                    {
                        attackDirection = 2;
                        counterAttackDirection = 3;
                        attackDir = Direction.Down;
                        counterAttackDir = Direction.Up;
                        unit.setCurrentFrame(1);
                        unit2.setCurrentFrame(10);
                    }
                    else if (unit.Position.Item2 > unit2.Position.Item2) //attack up
                    {
                        attackDirection = 3;
                        counterAttackDirection = 2;
                        attackDir = Direction.Up;
                        counterAttackDir = Direction.Down;
                        unit.setCurrentFrame(10);
                        unit2.setCurrentFrame(1);
                    }
                    #endregion
                    attackAnimation(attackDir, unit);
                    
                    bool attackType = false;

                    //find what attack type unit is making
                    attackType = findAttackType(unit);

                    int damageDealt = DamageCalculations.finalDamage(unit, unit2, attackType); //gets damage dealt
                    unit2.Hp = unit2.Hp - damageDealt; //updates hp

                    if (unit2.Alive && (GameFunction.isEnemyUnitInRange(graph, unit2, unit))) //if unit 2 is still alive, perform a counter attack
                    {
                        Thread.Sleep(750);
                        attackAnimation(counterAttackDir, unit2);
                        attackType = findAttackType(unit2); //gets attack type 
                        int damageTaken = DamageCalculations.finalDamage(unit2, unit, attackType); //damage dealt
                        unit.Hp = unit.Hp - damageTaken; //update hp
                    }
                    GameState.attackConfirmOpen = false;
                    setSelectedUnit(null, false);
                    turnState = TurnState.Wait;
                    break;
                case ButtonType.Move:           // if moved is clicked
                    if (button.Active)          // if unit hasn't already moved
                    {
                        turnState = TurnState.Move;
                        GameState.dropDownMenuOpen = false;   // close the dropdownmenu when selecting where to move
                    }
                    break;
                case ButtonType.Items:          // if item is clicked
                    turnState = TurnState.Items;
                    GameState.dropDownMenuOpen = false;
                    break;
                case ButtonType.Wait:           // if wait is clicked
                    turnState = TurnState.Wait;
                    GameState.dropDownMenuOpen = false;
                    setSelectedUnit(null, false);
                    button.Active = false;
                    break;

                default:
                    break;
            }
        }
    }
}